/* picplanner-window.c
 *
 * Copyright 2021 user
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "window/picplanner-window.h"
#include "window/overview-page/overview-view.h"
#include "window/sun-page/sun-view.h"
#include "window/moon-page/moon-view.h"
#include "window/milky-way-page/milky-way-view.h"
#include "calculations/calculations_sun.h"
#include "calculations/calculations_moon.h"
#include "calculations/calculations_milky_way.h"


struct _PicplannerWindow
{
  GtkApplicationWindow  parent_instance;

  /* Template widgets */
  GtkHeaderBar      *header_bar;
  GtkWidget         *search_bar;    /* Search bar for the location */
  GtkWidget         *search_entry;  /* The search entry inside the search bar */
  GtkWidget         *search_button; /* The search button in the header bar so show the search bar */
  GtkWidget         *overview_box;  /* The overview page */
  GtkWidget         *sun_box;       /* The sun page */
  GtkWidget         *moon_box;      /* The moon page */
  GtkWidget         *milky_way_box; /* The milky way page */
  GSettings         *settings;      /* All settings of PicPlanner */

  /* Variables to limit the amount of user searches on can make per second */
  gboolean          search_new;
  gint              search_count;
  guint             search_timeout_id;
};

G_DEFINE_TYPE (PicplannerWindow, picplanner_window, GTK_TYPE_APPLICATION_WINDOW)


static void
websearch (PicplannerWindow *window)
{
  (void) window;
  g_print("Searching...\n");
}


static gboolean
search_timeout_signal (gpointer user_data)
{
  PicplannerWindow *window = user_data;
  if (window->search_count > 0)
    {
      window->search_count = 0;
    }
  else
    {
      window->search_new = TRUE;
      if (*gtk_editable_get_text (GTK_EDITABLE (window->search_entry)) != '\0')
          websearch(window);
      g_source_remove (window->search_timeout_id);
    }

  return TRUE;
}


static void
search_text_changed (GtkWidget *self,
                     PicplannerWindow *window)
{
  (void) self;
  if (window->search_new)
    {
      window->search_new = FALSE;
      window->search_timeout_id = g_timeout_add (SEARCH_TIMEOUT_LENGTH,
                                                 search_timeout_signal,
                                                 window);
    }
  window->search_count++;
}


static void
input_changed (GtkWidget *self,
               gpointer user_data)
{
  (void) user_data;
  double *array_coordinates_sun;
  double *array_coordinates_moon;
  double *array_coordinates_milky_way;
  GDateTime *date_time;
  double longitude, latitude;

  longitude = picplanner_overview_get_longitude (PICPLANNER_OVERVIEW (self));
  latitude = picplanner_overview_get_latitude (PICPLANNER_OVERVIEW (self));
  date_time = picplanner_overview_get_date_time (PICPLANNER_OVERVIEW (self));

  array_coordinates_sun = picplanner_get_array_coordinates_sun (date_time, longitude, latitude);
  array_coordinates_moon = picplanner_get_array_coordinates_moon (date_time, longitude, latitude);
  array_coordinates_milky_way = picplanner_get_array_coordinates_milky_way (date_time, longitude, latitude);

  picplanner_overview_set_sun (date_time, array_coordinates_sun, 1);
}


static void
picplanner_window_class_init (PicplannerWindowClass *klass)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  g_type_ensure (PICPLANNER_OVERVIEW_TYPE);
  g_type_ensure (PICPLANNER_SUN_TYPE);
  g_type_ensure (PICPLANNER_MOON_TYPE);
  g_type_ensure (PICPLANNER_MILKYWAY_TYPE);

  gtk_widget_class_set_template_from_resource (widget_class, "/de/zwarf/picplanner/window/picplanner-window.ui");
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, header_bar);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, search_bar);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, search_entry);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, search_button);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, overview_box);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, sun_box);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, moon_box);
  gtk_widget_class_bind_template_child (widget_class, PicplannerWindow, milky_way_box);

  gtk_widget_class_bind_template_callback (widget_class, search_text_changed);
}


static void
picplanner_window_init (PicplannerWindow *window)
{
  gtk_widget_init_template (GTK_WIDGET (window));

  window->settings = g_settings_new ("de.zwarf.picplanner");

  g_object_bind_property (window->search_button, "active",
                          window->search_bar, "search-mode-enabled",
                          G_BINDING_BIDIRECTIONAL);

  g_signal_connect (G_OBJECT (window->overview_box), "input-changed", G_CALLBACK (input_changed), window);

  window->search_new = true;
  window->search_count = 0;
  window->search_timeout_id = 0;
}
